@@ -2,7 +2,7 @@
                           codegenerator.cpp  -  description
                              -------------------
     begin                : Die Jul 9 2002
-    copyright            : (C) 2002-2019 by Andre Simon
+    copyright            : (C) 2002-2020 by Andre Simon
     email                : a.simon@mailbox.org
  ***************************************************************************/
 
@@ -41,7 +41,6 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 #include "pangogenerator.h"
 #include "odtgenerator.h"
 #include "astyle/astyle.h"
-#include "astyle/ASStreamIterator.h"
 
 #if !defined (QT)
 #include "ansigenerator.h"
@@ -150,13 +149,17 @@ CodeGenerator::CodeGenerator ( highlight::OutputType type )
 
      terminatingChar ( '\0' ),
      formatter ( NULL ),
+     streamIterator ( NULL ),
      formattingEnabled ( false ),
      formattingPossible ( false ),
      validateInput ( false ),
      numberWrappedLines ( true ),
      resultOfHook(false),
      lineContainedTestCase(false),
+     lineContainedStmt(false),
      applySyntaxTestCase(false),
+     toggleDynRawString(false),
+
      keywordCase ( StringTools::CASE_UNCHANGED ),
      eolDelimiter ('\n'),
      outputType ( type )
@@ -167,9 +170,10 @@ CodeGenerator::CodeGenerator ( highlight::OutputType type )
 CodeGenerator::~CodeGenerator()
 {
     delete formatter;
+    delete streamIterator;
 
     resetSyntaxReaders();
-    
+
     for (unsigned int i=0; i<pluginChunks.size(); i++) {
         delete pluginChunks[i];
     }
@@ -284,7 +288,6 @@ bool CodeGenerator::getValidateInput()
     return validateInput;
 }
 
-
 void CodeGenerator::setNumberWrappedLines ( bool flag )
 {
     numberWrappedLines=flag;
@@ -439,9 +442,8 @@ void CodeGenerator::reset()
         vector<int> overrideStyleAttrs=currentSyntax->getOverrideStyleAttributes();
         docStyle.overrideAttributes(overrideStyleAttrs);
         if (overrideStyleAttrs.size())
-            disableStyleCache = true;    
+            disableStyleCache = true;
     }
-    
 }
 
 string CodeGenerator::getThemeInitError()
@@ -482,10 +484,16 @@ string CodeGenerator::getSyntaxCatDescription(){
     return (currentSyntax)? currentSyntax->getCategoryDescription(): "";
 }
 
-string CodeGenerator::getThemeCatDescription(){
+string CodeGenerator::getThemeCatDescription()
+{
     return docStyle.getCategoryDescription();
 }
 
+float CodeGenerator::getThemeContrast()
+{
+    return docStyle.getContrast();
+}
+
 unsigned int CodeGenerator::getLineNumber()
 {
     return lineNumber;
@@ -494,9 +502,9 @@ unsigned int CodeGenerator::getLineNumber()
 bool CodeGenerator::readNewLine ( string &newLine )
 {
     bool eof=false;
-    
+
     if ( lineIndex ) terminatingChar=newLine[lineIndex-1];
-    
+
     while (!eof && startLineCntCurFile>0) {
         if ( formattingPossible && formattingEnabled ) {
             eof=!formatter->hasMoreLines();
@@ -533,7 +541,7 @@ void CodeGenerator::matchRegex ( const string &line, State skipState)
         if (regexElem->constraintLineNum && regexElem->constraintLineNum != lineNumber) {
             continue;
         }
-        
+
         if (regexElem->constraintFilename.size() && regexElem->constraintFilename != inFile) {
             continue;
         }
@@ -544,22 +552,15 @@ void CodeGenerator::matchRegex ( const string &line, State skipState)
         for( ; cur != end; ++cur )  {
             groupID = ( regexElem->capturingGroup<0 ) ? cur->size()-1 : regexElem->capturingGroup;
             matchBegin = cur->position(groupID);
-        
+
             regexGroups.insert (
                 make_pair ( matchBegin + 1, ReGroup ( regexElem->open, cur->length(groupID), regexElem->kwClass, regexElem->langName ) ) );
-            
-            //Only return first match
-            //if (regexElem->kwClass==6)
-              //  break;
 
             // priority regex (match required)
             if (regexElem->priority) {
                 return;
             }
-
         }
-        
-        // prio check here if match not necessary
     }
 }
 
@@ -567,11 +568,11 @@ unsigned char CodeGenerator::getInputChar()
 {
     // end of line?
     if ( lineIndex == line.length() ) {
-        
+
         //more testing required:
         if (outputType==ESC_TRUECOLOR || outputType==ESC_XTERM256)
             lastLineLength=StringTools::utf8_strlen(line);
-        
+
         bool eof=false;
         if ( preFormatter.isEnabled() ) {
             if ( !preFormatter.hasMoreLines() ) {
@@ -593,24 +594,25 @@ unsigned char CodeGenerator::getInputChar()
             numberCurrentLine = true;
         }
         lineIndex=0;
-        
+
         if (!lineContainedTestCase && applySyntaxTestCase){
             stateTraceTest = stateTraceCurrent;
             stateTraceCurrent.clear();
-        } 
-        
+        }
+
         lineContainedTestCase=false;
-            
+        lineContainedStmt=false;
         matchRegex ( line );
-        stateTrace.clear();
+
         return ( eof ) ?'\0':'\n';
     }
 
     return line[lineIndex++];
 }
 
-/** changing this method requires regression testing with nested syntax files (HTML+PHP+JS+CSS, Coffeescript with block regex, Pas + ASM) 
-    especially nested syntax in one line
+/** changing this method requires regression testing with nested syntax files (HTML+PHP+JS+CSS,
+ *  Coffeescript with block regex, Pas + ASM)
+ *  especially nested syntax in one line
  */
 State CodeGenerator::getCurrentState (State oldState)
 {
@@ -632,24 +634,23 @@ State CodeGenerator::getCurrentState (State oldState)
 
     if ( c==' ' || c=='\t' ) {
         token= c;
-        return _WS;
+        return _WS;    // White space
     }
-    
-    //TODO add control flag
+
     if ( applySyntaxTestCase && ( c=='^' || c=='<') && (oldState == ML_COMMENT || oldState==SL_COMMENT)  ) {
         token= c;
         return _TESTPOS;
     }
-        
+
     // at this position the syntax change takes place
     if (lineIndex >= syntaxChangeIndex-1 || syntaxChangeLineNo < lineNumber){
-        loadEmbeddedLang(embedLangDefPath);  // load new syntax                     
+        loadEmbeddedLang(embedLangDefPath);  // load new syntax
         matchRegex(line);                    // recognize new patterns in the (remaining) line
         syntaxChangeIndex = syntaxChangeLineNo = UINT_MAX;
     }
 
 SKIP_EMBEDDED:
-    
+
     // Test if a regular expression was found at the current position
     if ( !regexGroups.empty() ) {
         if ( regexGroups.count ( lineIndex ) ) {
@@ -659,14 +660,14 @@ SKIP_EMBEDDED:
             if ( regexGroups[oldIndex].length>1 ) lineIndex+= regexGroups[oldIndex].length-1;
 
             if ( regexGroups[oldIndex].state==EMBEDDED_CODE_BEGIN ) {
-                //do not handle a nested section if the syntax is marked as "sealed" 
+                //do not handle a nested section if the syntax is marked as "sealed"
                 if (embedLangDefPath.length()==0 || currentSyntax->allowsInnerSection(embedLangDefPath) ) {
                     embedLangDefPath = currentSyntax->getNewPath(regexGroups[oldIndex].name);
-                    //remember position 
+                    //remember position
                     syntaxChangeIndex = lineIndex+2;
                     syntaxChangeLineNo = lineNumber;
                 }
-                
+
                 // repeat parsing of this line without nested state recognition to highlight opening delimiter in the host syntax
                 matchRegex(line, EMBEDDED_CODE_BEGIN);
                 lineIndex = oldIndex;
@@ -676,7 +677,7 @@ SKIP_EMBEDDED:
             if ( regexGroups[oldIndex].state==IDENTIFIER_BEGIN || regexGroups[oldIndex].state==KEYWORD ) {
                 string reservedWord= ( currentSyntax->isIgnoreCase() ) ? StringTools::change_case ( token ) :token;
                 currentKeywordClass=currentSyntax->getKeywordListGroup ( reservedWord ); //check in lists (no regex)
-                
+
                 if ( !currentKeywordClass && regexGroups[oldIndex].state==KEYWORD ){
                     currentKeywordClass = regexGroups[oldIndex].kwClass;
                 }
@@ -710,15 +711,29 @@ State CodeGenerator::validateState(State newState, State oldState)
 
         resultOfHook = res.size()>=1;
         if (resultOfHook) {
+
+            if (currentSyntax->requiresParamUpdate()) {
+
+                 if ( currentSyntax->getOverrideConfigVal("state.string.raw")=="true"){
+                     toggleDynRawString=true; // reset to false in string state fct
+                 }
+                 if ( currentSyntax->getOverrideConfigVal("format.maskws")=="true") {
+                     maskWs=true;
+                 }
+                 if ( currentSyntax->getOverrideConfigVal("format.spacer").size()) {
+                     spacer=currentSyntax->getOverrideConfigVal("format.spacer");
+                 }
+            }
+
             State validatedState = (State)res[0].asInteger();
             if ( validatedState== _REJECT) {
                 // proceed using only the first character of the token
                 // TODO evaluate if token clear would be better
-                if (res.size()==1) { 
+                if (res.size()==1) {
                     lineIndex -= (token.length() -1);
                     token=token.substr(0, 1);
                 }
-                
+
                 //experimental for slim.lang: evaluate second return arg after _REJECT
                 if (res.size()>=2) {
                     lineIndex -= (token.length() );
@@ -727,14 +742,12 @@ State CodeGenerator::validateState(State newState, State oldState)
                 }
                 return oldState;
             }
-            stateTrace.push_back(validatedState);
-            if (stateTrace.size()>200) stateTrace.erase(stateTrace.begin(), stateTrace.begin() + 100 );
+
             return validatedState;
         }
     }
     resultOfHook  = false;
-    stateTrace.push_back(newState);
-    if (stateTrace.size()>200) stateTrace.erase(stateTrace.begin(), stateTrace.begin() + 100 );    
+
     return newState;
 }
 
@@ -758,40 +771,30 @@ void CodeGenerator::maskString ( ostream& ss, const string & s )
     for ( unsigned int i=0; i< s.length(); i++ ) {
         ss << maskCharacter ( s[i] );
     }
-    
+
     // The test markers position should also be deternmined by calculating the code points
     if ( applySyntaxTestCase ) {
-            
+
         PositionState ps(currentState, getCurrentKeywordClassId(), false);
-        
-        int slen = encoding=="utf-8" ? StringTools::utf8_strlen(s) : s.length();  
+
+        int slen = encoding=="utf-8" ? StringTools::utf8_strlen(s) : s.length();
         for (int i=0; i< slen; i++ ) {
             stateTraceCurrent.push_back(ps);
         }
-        if (stateTraceCurrent.size()>200) 
-            stateTraceCurrent.erase(stateTraceCurrent.begin(), stateTraceCurrent.begin() + 100 ); 
-    } 
-    
+        if (stateTraceCurrent.size()>200)
+            stateTraceCurrent.erase(stateTraceCurrent.begin(), stateTraceCurrent.begin() + 100 );
+    }
 }
 
 
-Diluculum::LuaValueList CodeGenerator::callDecorateFct(const string&token)
+Diluculum::LuaValueList CodeGenerator::callDecorateFct(const string& token)
 {
+
     Diluculum::LuaValueList params;
     params.push_back(Diluculum::LuaValue(token));
     params.push_back(Diluculum::LuaValue(currentState));
     params.push_back(Diluculum::LuaValue(currentKeywordClass));
-    string trace(";");
-    if (stateTrace.size()>1){
-        for (size_t i=0; i<stateTrace.size()-1;i++){
-            trace += std::to_string (stateTrace[i]);
-            trace += ";";
-        }
-    }
-    
-    //std::cerr <<"TRC1: "<<trace<<"\n";
-    
-    params.push_back(Diluculum::LuaValue(trace));
+    params.push_back(Diluculum::LuaValue(lineContainedStmt));
 
     return currentSyntax->getLuaState()->call ( *currentSyntax->getDecorateFct(),
             params,"getDecorateFct call")  ;
@@ -814,6 +817,11 @@ void CodeGenerator::printMaskedToken (bool flushWhiteSpace, StringTools::Keyword
         maskString ( *out, caseToken );
     }
 
+    // check this *after* the decorate call
+    if (   currentState == STANDARD || currentState == KEYWORD || currentState == NUMBER
+        || currentState == STRING || currentState == IDENTIFIER_BEGIN) {
+        lineContainedStmt = true;
+    }
     token.clear();
 }
 
@@ -822,8 +830,7 @@ bool CodeGenerator::styleFound()
     return docStyle.found();
 }
 
-bool CodeGenerator::printIndexFile ( const vector<string> &fileList,
-                                     const string &outPath )
+bool CodeGenerator::printIndexFile ( const vector<string> &fileList, const string &outPath )
 {
     return true;
 }
@@ -920,7 +927,7 @@ void CodeGenerator::setIndentationOptions (const vector<string>& options){
         string arg;
         for (unsigned int i=0; i<options.size(); i++) {
             arg=options[i];
-            
+
             if (isOption(arg, "mode=cs"))
             {
                 formatter->setSharpStyle();
@@ -970,7 +977,6 @@ void CodeGenerator::setIndentationOptions (const vector<string>& options){
                     tabNum = atoi(tabNumParam.c_str());
                 if (tabNum >= 2 && tabNum <= 20)
                     formatter->setForceTabXIndentation(tabNum);
-                
             }
             else if (isOption(arg, "indent=force-tab-x"))
             {
@@ -1323,9 +1329,9 @@ LoadResult CodeGenerator::loadLanguage ( const string& langDefPath, bool embedde
     if (!embedded) {
         while (!nestedLangs.empty()) {
             nestedLangs.pop();
-        }   
+        }
     }
-    
+
     bool reloadNecessary= currentSyntax ? currentSyntax->needsReload ( langDefPath ): true;
     LoadResult result=LOAD_OK;
     if ( reloadNecessary ) {
@@ -1355,7 +1361,7 @@ LoadResult CodeGenerator::loadLanguage ( const string& langDefPath, bool embedde
                 openTags.push_back ( getKeywordOpenTag ( i ) );
                 closeTags.push_back ( getKeywordCloseTag ( i ) );
             }
-            
+
             //test balloon
             string overrideSpacer(currentSyntax->getOverrideConfigVal("spacer"));
             if (!overrideSpacer.empty()) {
@@ -1365,7 +1371,7 @@ LoadResult CodeGenerator::loadLanguage ( const string& langDefPath, bool embedde
             if (!overrideMaskWS.empty()) {
                 maskWs = overrideMaskWS=="true";
             }
-            
+
         }
     }
     return result;
@@ -1426,9 +1432,9 @@ bool CodeGenerator::validateInputStream()
 }
 
 void CodeGenerator::applyPluginChunk(const string& fctName, string *result, bool *keepDefault) {
-    
+
     if (currentSyntax && pluginChunks.size()) {
-    
+
         Diluculum::LuaState luaState;
 
         Diluculum::LuaValueList chunkParams;
@@ -1436,10 +1442,10 @@ void CodeGenerator::applyPluginChunk(const string& fctName, string *result, bool
         for (unsigned int i=0; i<pluginChunks.size(); i++) {
             luaState.call(*pluginChunks[i], chunkParams, "format user function");
         }
-        
+
         if (luaState.globals().count(fctName)) {
             Diluculum::LuaFunction* documentFct=new Diluculum::LuaFunction(luaState[fctName].value().asFunction());
-        
+
             luaState["HL_INPUT_FILE"] = luaState["HL_PLUGIN_PARAM"] = pluginParameter;
             luaState["HL_OUTPUT"] = outputType;
             luaState["HL_FORMAT_HTML"]=HTML;
@@ -1454,20 +1460,20 @@ void CodeGenerator::applyPluginChunk(const string& fctName, string *result, bool
             luaState["HL_FORMAT_BBCODE"]=BBCODE;
             luaState["HL_FORMAT_PANGO"]=PANGO;
             luaState["HL_FORMAT_ODT"]=ODTFLAT;
-                        
+
             Diluculum::LuaValueList params;
             Diluculum::LuaValueMap options;
-            options[Diluculum::LuaValue("title")] =  Diluculum::LuaValue( docTitle );   
-            options[Diluculum::LuaValue("encoding")] =  Diluculum::LuaValue(encoding);   
-            options[Diluculum::LuaValue("fragment")] =  Diluculum::LuaValue(fragmentOutput);   
-            options[Diluculum::LuaValue("font")] =  Diluculum::LuaValue(getBaseFont());   
-            options[Diluculum::LuaValue("fontsize")] =  Diluculum::LuaValue(getBaseFontSize());   
+            options[Diluculum::LuaValue("title")] =  Diluculum::LuaValue( docTitle );
+            options[Diluculum::LuaValue("encoding")] =  Diluculum::LuaValue(encoding);
+            options[Diluculum::LuaValue("fragment")] =  Diluculum::LuaValue(fragmentOutput);
+            options[Diluculum::LuaValue("font")] =  Diluculum::LuaValue(getBaseFont());
+            options[Diluculum::LuaValue("fontsize")] =  Diluculum::LuaValue(getBaseFontSize());
 
             params.push_back(inputFilesCnt);
             params.push_back(processedFilesCnt);
             params.push_back(options);
-            
-            Diluculum::LuaValueList res=luaState.call ( *documentFct, params, fctName+" call")  ;
+
+            Diluculum::LuaValueList res=luaState.call ( *documentFct, params, fctName+" call");
             if (res.size()>=1) {
                 *keepDefault=false;
                 *result = res[0].asString();
@@ -1483,33 +1489,33 @@ void CodeGenerator::printHeader()
 {
     bool keepDefaultHeader=true;
     string pluginHeader;
-    
+
     processedFilesCnt++;
-    
+
     applyPluginChunk("DocumentHeader", &pluginHeader, &keepDefaultHeader);
 
     if ( ! fragmentOutput && keepDefaultHeader)
         *out << getHeader();
-    
-    *out << pluginHeader; 
-   
+
+    *out << pluginHeader;
+
     if ( !fragmentOutput || keepInjections)
         *out << currentSyntax->getHeaderInjection();
 }
 
 void CodeGenerator::printFooter()
 {
-    
+
     bool keepDefaultFooter=true;
     string pluginFooter;
-    
+
     applyPluginChunk("DocumentFooter", &pluginFooter, &keepDefaultFooter);
-    
+
     if ( !fragmentOutput || keepInjections)
         *out << currentSyntax->getFooterInjection();
 
-    *out << pluginFooter; 
-    
+    *out << pluginFooter;
+
     if ( ! fragmentOutput && keepDefaultFooter )
         *out << getFooter();
 }
@@ -1527,7 +1533,7 @@ ParseError CodeGenerator::generateFile ( const string &inFileName,
 
     inFile=inFileName;
     outFile=outFileName;
-        
+
     in = ( inFileName.empty() ? &cin :new ifstream ( inFileName.c_str() ) );
 
     if ( validateInput )
@@ -1545,9 +1551,7 @@ ParseError CodeGenerator::generateFile ( const string &inFileName,
     }
 
     if ( error==PARSE_OK ) {
-        if ( formatter != NULL ) {
-            formatter->init ( new astyle::ASStreamIterator ( in ) );
-        }
+        initASStream();
         currentSyntax->setInputFileName(inFile);
         printHeader();
         printBody();
@@ -1581,9 +1585,8 @@ string CodeGenerator::generateString ( const string &input )
         return "";
     }
 
-    if ( formatter != NULL ) {
-        formatter->init ( new astyle::ASStreamIterator ( in ) );
-    }
+    initASStream();
+
     printHeader();
     printBody();
     printFooter();
@@ -1598,6 +1601,14 @@ string CodeGenerator::generateString ( const string &input )
     return result;
 }
 
+void CodeGenerator::initASStream() {
+    if ( formatter != NULL ) {
+        if (streamIterator) delete streamIterator;
+        streamIterator =  new astyle::ASStreamIterator ( in );
+        formatter->init ( streamIterator );
+    }
+}
+
 string CodeGenerator::generateStringFromFile ( const string &inFileName )
 {
 
@@ -1608,7 +1619,7 @@ string CodeGenerator::generateStringFromFile ( const string &inFileName )
     reset();
 
     inFile = inFileName;
-    
+
     in = new ifstream ( inFileName.c_str() );
     out = new ostringstream ();
 
@@ -1620,11 +1631,10 @@ string CodeGenerator::generateStringFromFile ( const string &inFileName )
         return "ERROR: detected binary input";
     }
 
-    if ( formatter != NULL ) {
-        formatter->init ( new astyle::ASStreamIterator ( in ) );
-    }
+    initASStream();
+
     currentSyntax->setInputFileName(inFile);
-    
+
     printHeader();
     printBody();
     printFooter();
@@ -1694,9 +1704,9 @@ void CodeGenerator::processRootState()
 {
     bool eof=false,
          firstLine=true; // avoid newline before printing the first output line
-         
+
     applySyntaxTestCase = inFile.find("syntax_test_")!=string::npos;
-    
+
     if ( currentSyntax->highlightingDisabled() ) {
         string line;
         while ( getline ( *in, line ) && lineNumber < maxLineCnt ) {
@@ -1716,10 +1726,8 @@ void CodeGenerator::processRootState()
     }
 
     State state=STANDARD;
+    openTag ( STANDARD );
 
-    if (outputType!=ESC_TRUECOLOR && outputType!=ESC_XTERM256)
-        openTag ( STANDARD );
-    
     do {
         // determine next state
         state= getCurrentState(STANDARD);
@@ -1773,12 +1781,11 @@ void CodeGenerator::processRootState()
             openTag ( STANDARD );
             break;
         case _EOL:
-
             // XTERM256 fix (issue with less cmd)
             if  (!firstLine || showLineNumbers) {
                 closeTag ( STANDARD );
             }
-            insertLineNumber ( !firstLine );
+            insertLineNumber(!firstLine);
             if (!firstLine || showLineNumbers) {
                 flushWs(5);
                 stateTraceCurrent.clear();
@@ -1793,16 +1800,11 @@ void CodeGenerator::processRootState()
             processWsState();
             break;
         default:
-            
-         // breaks Unicode sequences:
-         //   if ((outputType==ESC_TRUECOLOR || outputType==ESC_XTERM256) && token.size())
-           //     openTag ( STANDARD );
-
-            printMaskedToken ();
+            printMaskedToken();
             break;
         }
     } while ( !eof );
-    
+
     if (token.size() || lineNumber>1 || (outputType!=ESC_TRUECOLOR && outputType!=ESC_XTERM256))
         closeTag ( STANDARD );
 
@@ -1837,7 +1839,7 @@ bool CodeGenerator::processSyntaxChangeState(State myState)
             }
             matchRegex(line, EMBEDDED_CODE_BEGIN); // match remaining line using the host syntax
         }
-        
+
         printMaskedToken ( newState!=_WS );
 
         newState= getCurrentState(myState);
@@ -1884,7 +1886,7 @@ bool CodeGenerator::processKeywordState ( State myState )
         case _EOL:
             insertLineNumber();
             exitState=true;
-            
+
             break;
         case _EOF:
             eof = true;
@@ -1992,7 +1994,7 @@ bool CodeGenerator::processMultiLineCommentState()
     } while ( !exitState  &&  !eof );
 
     closeTag ( ML_COMMENT );
-   
+
     if (containedTestCase){
         stateTraceCurrent.clear();
     }
@@ -2035,18 +2037,18 @@ bool CodeGenerator::processSingleLineCommentState()
             printMaskedToken();
             containedTestCase=true;
             break;
-     
+
         default:
             break;
         }
     } while ( !exitState  &&  !eof );
 
     closeTag ( SL_COMMENT );
-    
+
     if (containedTestCase) {
         stateTraceCurrent.clear();
     }
-    
+
     return eof;
 }
 
@@ -2069,13 +2071,13 @@ bool CodeGenerator::processDirectiveState()
             break;
         case _EOL:
             printMaskedToken();
-            
+
             if ( preFormatter.isEnabled() && preFormatter.isWrappedLine ( lineNumber-1 ) ) {
                 exitState=false;
             } else {
                 if (currentSyntax->getContinuationChar()!=0x13){
                     exitState= ( terminatingChar!=currentSyntax->getContinuationChar() );
-                } 
+                }
             }
             if ( !exitState ) wsBuffer += closeTags[DIRECTIVE];
             insertLineNumber();
@@ -2121,7 +2123,7 @@ bool CodeGenerator::processStringState ( State oldState )
     string openDelim=token;
 
     //Raw String by definition:
-    bool isRawString=currentSyntax->delimiterIsRawString(openDelimID);
+    bool isRawString=currentSyntax->delimiterIsRawString(openDelimID) || toggleDynRawString;
 
     // Test if character before string open delimiter token equals to the
     // raw string prefix (Example: r" ", r""" """ in Python)
@@ -2162,7 +2164,7 @@ bool CodeGenerator::processStringState ( State oldState )
         case STRING:
             // if there exist multiple string delimiters, close string if
             // current delimiter is equal to the opening delimiter
-            exitState=currentSyntax->delimiterIsDistinct(currentSyntax->getOpenDelimiterID ( token, STRING  ))&&token==openDelim;
+            exitState=currentSyntax->delimiterIsDistinct(currentSyntax->getOpenDelimiterID ( token, STRING  )) && token==openDelim;
             printMaskedToken();
             break;
         case ESC_CHAR:
@@ -2183,7 +2185,7 @@ bool CodeGenerator::processStringState ( State oldState )
             openTag ( myState );
             returnedFromOtherState=true;
             break;
-    
+
         case _EOF:
             eof = true;
             break;
@@ -2195,6 +2197,8 @@ bool CodeGenerator::processStringState ( State oldState )
 
     closeTag ( myState );
 
+    toggleDynRawString = false;
+
     return eof;
 }
 
@@ -2304,7 +2308,7 @@ void CodeGenerator::processWsState()
     int cntWs=0;
     lineIndex--;
     PositionState ps(currentState, 0, true);
-            
+
     while ( line[lineIndex]==' ' || line[lineIndex]=='\t' ) {
         ++cntWs;
         ++lineIndex;
@@ -2315,7 +2319,8 @@ void CodeGenerator::processWsState()
         if ( excludeWs && styleID!=_UNKNOWN ) {
             *out << closeTags[styleID];
         }
-        *out << maskWsBegin ;
+
+        *out << maskWsBegin;
         for ( int i=0; i<cntWs; i++ ) {
             *out <<  spacer;
             if (applySyntaxTestCase){
@@ -2327,10 +2332,10 @@ void CodeGenerator::processWsState()
             *out << openTags[styleID];
         }
     } else {
-    
+
         *out << spacer; //Bugfix fehlender Space nach Strings
         if (applySyntaxTestCase){
-            stateTraceCurrent.push_back(ps);            
+            stateTraceCurrent.push_back(ps);
         }
     }
     token.clear();
@@ -2338,25 +2343,25 @@ void CodeGenerator::processWsState()
 
 void CodeGenerator::flushWs(int arg)
 {
-     PositionState ps(currentState, 0, true);
-     //workaround condition
-     for ( size_t i=0; i<wsBuffer.size() && ((arg > 3) || ( (arg<4) && lineIndex>1)) && applySyntaxTestCase ; i++ ) {
+    PositionState ps(currentState, 0, true);
+    //workaround condition
+    for ( size_t i=0; i<wsBuffer.size() && ((arg > 3) || ( (arg<4) && lineIndex>1)) && applySyntaxTestCase ; i++ ) {
         stateTraceCurrent.push_back(ps);
-        //std::cerr <<"\nflush >"<<wsBuffer<<"< arg:"<<arg;           
+        //std::cerr <<"\nflush >"<<wsBuffer<<"< arg:"<<arg;
     }
-     
-     //fix canvas whitespace
-    if (outputType==ESC_XTERM256 || outputType==ESC_TRUECOLOR){
+
+    //fix canvas whitespace
+    if (wsBuffer.length() && (outputType==ESC_XTERM256 || outputType==ESC_TRUECOLOR) ){
         *out<< maskWsBegin;
     }
-     
+
     *out<<wsBuffer;
     wsBuffer.clear();
 }
 
 string CodeGenerator::getTestcaseName(State s, unsigned int kwClass) {
     switch (s) {
-        
+
         case STANDARD:
             return STY_NAME_STD;
         case STRING:
@@ -2380,10 +2385,10 @@ string CodeGenerator::getTestcaseName(State s, unsigned int kwClass) {
         case _WS:
             return "ws";
         case KEYWORD: {
-            
+
             if (!kwClass)
                 return "ws";
-            
+
             char kwName[5] = {0};
             snprintf(kwName, sizeof(kwName), "kw%c", ('a'+kwClass-1));
             return string(kwName);
@@ -2402,39 +2407,34 @@ void CodeGenerator::printTrace(const string &s){
     for (unsigned int i=0; i< stateTraceTest.size(); i++) {
         std::cout<<" "<<stateTraceTest[i].state;
     }
-    /*
-    for (unsigned int i=0; i< stateTrace.size(); i++) {
-        std::cout<<" "<<stateTrace[i];
-    }
-   */
     std::cout<<"\n";
 }
 
 //column: lineIndex (not a UTF-8 validated string position)
 void CodeGenerator::runSyntaxTestcases(unsigned int column){
-    
+
     if (encoding=="utf-8")
         column = StringTools::utf8_strlen(line.substr(0, column));
-    
+
     unsigned int assertGroup=0;
     size_t typeDescPos=line.find_first_not_of("\t ^", lineIndex);
     State assertState=_UNKNOWN;
     bool negation=false;
     bool testFailed=false;
-   
+
     ostringstream errMsg;
     string prefix;
     //printTrace("trace 2");
-    
+
     if (typeDescPos!=string::npos) {
-    
+
         if (line[typeDescPos]=='~') {
-        
+
             negation=true;
             prefix="~";
             ++typeDescPos;
         }
-        
+
         if (line.find(STY_NAME_NUM, typeDescPos)==typeDescPos)
             assertState=NUMBER;
         else if (line.find(STY_NAME_STR, typeDescPos)==typeDescPos)
@@ -2457,38 +2457,38 @@ void CodeGenerator::runSyntaxTestcases(unsigned int column){
             assertState=STANDARD;
         else if (line.find(STY_NAME_DST, typeDescPos)==typeDescPos)
             assertState=DIRECTIVE_STRING;
-        
+
         else if (line.find("kw", typeDescPos)==typeDescPos) {
             assertState=KEYWORD;
             if (isalpha(line[typeDescPos+2]))
                 assertGroup=line[typeDescPos+2] - 'a' +1;
         }
-    
+
        if (   (assertState!=_WS && stateTraceTest[column].state != assertState && !stateTraceTest[column].isWhiteSpace )
             || (assertState==_WS && !stateTraceTest[column].isWhiteSpace)
             || assertGroup != stateTraceTest[column].kwClass) {
-            
+
             testFailed=!negation;
-       
+
         } else if (negation ) {
-            
-            //TODO Fix ~ws 
+
+            //TODO Fix ~ws
             if (assertState!=_WS  && !stateTraceTest[column].isWhiteSpace )
                 testFailed=true;
-            
+
         }
 
         if (testFailed) {
-            errMsg << inFile << " line " << lineNumber << ", column "<< column 
-                    << ": got " << getTestcaseName(stateTraceTest[column].state, stateTraceTest[column].kwClass)  
+            errMsg << inFile << " line " << lineNumber << ", column "<< column
+                    << ": got " << getTestcaseName(stateTraceTest[column].state, stateTraceTest[column].kwClass)
                     << " instead of " << prefix << getTestcaseName(assertState, assertGroup);
-        
-            failedPosTests.push_back(errMsg.str());    
+
+            failedPosTests.push_back(errMsg.str());
         }
-        
+
     }
-    
-    lineContainedTestCase=true; 
+
+    lineContainedTestCase=true;
 }
 
 
@@ -2594,18 +2594,18 @@ bool CodeGenerator::printPersistentState ( const string &outFile )
 
     ofstream pluginOutFile( outFile.c_str());
     if ( !pluginOutFile.fail() ) {
-        
+
         pluginOutFile   <<"Description=\"Plugin generated by highlight using the --two-pass option\"\n\n"
                         <<"Categories = {\"two-pass\" }\n\n"
                         <<"function syntaxUpdate(desc)\n\n";
-                        
+
         pluginOutFile << currentSyntax->getPersistentHookConditions();
-     
+
         for (auto snippet: currentSyntax->getPersistentSnippets())
-        {   
+        {
             pluginOutFile << snippet <<"\n\n";
         }
-        
+
         pluginOutFile<<"end\n\n"
                      <<"Plugins={\n"
                      <<"  { Type=\"lang\", Chunk=syntaxUpdate }\n"
@@ -2613,7 +2613,7 @@ bool CodeGenerator::printPersistentState ( const string &outFile )
     } else {
         return false;
     }
-    
+
     return true;
 }
 
@@ -2623,18 +2623,18 @@ string CodeGenerator::readUserStyleDef()
     if ( !styleInputPath.empty() ) {
         ifstream userStyleDef ( styleInputPath.c_str() );
         if ( userStyleDef ) {
-            ostr 	<< "\n" << styleCommentOpen
-                    << " Content of " << styleInputPath
-                    << ": " <<styleCommentClose << "\n";
+            ostr << "\n" << styleCommentOpen
+                << " Content of " << styleInputPath
+                << ": " <<styleCommentClose << "\n";
             string line;
             while ( getline ( userStyleDef, line ) ) {
                 ostr << line << "\n";
             }
             userStyleDef.close();
         } else {
-            ostr 	<< styleCommentOpen
-                    << " ERROR: Could not include " << styleInputPath
-                    << "." << styleCommentClose << "\n";
+            ostr << styleCommentOpen
+                << " ERROR: Could not include " << styleInputPath
+                << "." << styleCommentClose << "\n";
         }
     }
 
@@ -2656,7 +2656,7 @@ bool CodeGenerator::initPluginScript(const string& script)
 
         userScriptError="";
         Diluculum::LuaState ls;
-        
+
         ls.doFile (script);
         int listIdx=1;
 
@@ -2680,7 +2680,7 @@ bool CodeGenerator::initPluginScript(const string& script)
                     addUserChunk(ls["Plugins"][listIdx]["Chunk"].value().asFunction());
                 }
             }
-            
+
             listIdx++;
         }
     }  catch (Diluculum::LuaError &err) {
