@@ -126,7 +126,7 @@ void HtmlGenerator::printBody()
                  <<((quoteFont)?"'":"")
                  << this->getBaseFont()
                  <<((quoteFont)?"'":"")
-                 <<";\">";
+                 <<";white-space: pre-wrap;\">";
         }
     }
     if ( showLineNumbers && orderedList ) {
@@ -224,7 +224,7 @@ string HtmlGenerator::getGeneratorComment()
 
 string HtmlGenerator::getStyleDefinition()
 {
-    if ( styleDefinitionCache.empty() ) {
+    if (disableStyleCache || styleDefinitionCache.empty() ) {
         bool quoteFont=getBaseFont().find_first_of(",'")==string::npos;
         ostringstream os;
         
@@ -255,7 +255,7 @@ string HtmlGenerator::getStyleDefinition()
            << "; font-size:" << this->getBaseFontSize();
 
         os << "pt; font-family:"<<((quoteFont)?"'":"") << getBaseFont() << ((quoteFont)?"'":"")
-           << ";}\n";
+           << "; white-space: pre-wrap; }\n";
 
         os  << getAttributes ( STY_NAME_NUM, docStyle.getNumberStyle() )
             << getAttributes ( STY_NAME_ESC, docStyle.getEscapeCharStyle() )
