@@ -36,7 +36,10 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "syntaxreader.h"
 #include "themereader.h"
+
 #include "astyle/astyle.h"
+#include "astyle/ASStreamIterator.h"
+
 #include "preformatter.h"
 #include "enums.h"
 #include "stringtools.h"
@@ -428,15 +431,18 @@ public:
     /** \return Syntaxreader description */
     string getSyntaxDescription();
 
-        /** \return Encoding hint */
+    /** \return Encoding hint */
     string getSyntaxEncodingHint();
-    
+
     /** \return Theme description */
     string getThemeDescription();
 
     /** \return Syntaxreader category description */
     string getSyntaxCatDescription();
 
+    /** \return Encoding contrast of Canvas and Default colours */
+    float getThemeContrast();
+
     /** \return Theme category description */
     string getThemeCatDescription();
 
@@ -632,9 +638,6 @@ protected:
     /** Current state*/
     State currentState;
 
-    /** history of preceding states in the current line of input code (max 200 entries) */
-    vector<State> stateTrace;
-    
     /** keyword class id, used to apply the corresponding keyword style*/
     unsigned int currentKeywordClass;
 
@@ -802,13 +805,13 @@ private:
     
     /** Flag to test if trailing newline should be printed */
     int noTrailingNewLine;
-
     
     /**last character of the last line*/
     unsigned char terminatingChar;
 
     /** Class for reformatting */
     astyle::ASFormatter *formatter;
+    astyle::ASStreamIterator *streamIterator;
 
     /** Flag to test if formatting is enabled with current input document*/
     bool formattingEnabled;
@@ -822,14 +825,17 @@ private:
     /** Flag if wrapped lines should receive unique line numbers as well */
     bool numberWrappedLines;
 
-
     /** indicator if current state was set by Lua hook function */
     bool resultOfHook;
     
     bool lineContainedTestCase;
-    
+
+    bool lineContainedStmt;
+
     bool applySyntaxTestCase;
 
+    bool toggleDynRawString;
+
     /** flag which determines keyword output (unchangeed, uppercase, lowercase)*/
     StringTools::KeywordCase keywordCase;
 
@@ -947,6 +953,8 @@ private:
     
     void applyPluginChunk(const string& fctName, string *result, bool *keepDefault);
     
+    void initASStream();
+    
     static vector<Diluculum::LuaFunction*> pluginChunks;
 };
 
